/* TwigMan - An SDL demonstration for CSCI 321.
   Version 1.0.1
   Written by Dr. Evan L. Schemm
              Lake Superior State University
              School of Mathematics and Computer Science

   Please send questions, comments, suggestions or (gasp!) bugfixes to:
          elschemm@furball.schemm.lssu.edu

   Copyright 2007 by LSSU School of MA/CS

   Licensed under GPL.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.


   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


/*
  TwigMan is the third demonstration program produced as a talking point for
  CSCI 321, Computer Graphics, at Lake Superior State University.

  TwigMan started as a fairly simple jumping game meant to demonstate gravity
  and platform games.  Because of chemicals released into the building air
  ducts, it morphed into this.  

  The goal of TwigMan is for Twig to re-collect the cable testers at the 
  end of CSCI 221 (Computer Networking).  Sadly, over the course of the years,
  a number of them have 'disappeared'.  Interestingly, the students do have 
  a remarkable tendency to leave their textbooks in the lab.  And of course, 
  Dr. Terwilliger is never far from his beverage of choice.  I'm still not
  entirely certain how he convinced the University to move the vending machine
  outside the lab.

  The standard keys for TwigMan are as follows:
  5 - Insert Coin (MAME standard)
  1 - Player 1 start (MAME standard)
  Left - Move left
  Right - Move right
  Up - Jump
  F11 - Toggle Fullscreen
  P - Pause (only during normal gameplay)

  These keys can be re-mapped to any keys on the keyboard by changing the 
  default values in the file 'twigman.conf'.


  With TwigPower, TwigMan can double (and triple, ...) jump.  He is also immune
  to the FabbriCar and Falling Boger.

  You have to land on the students head to squash them (and get the points).
  No, you don't get any cable testers they stole back (already sold on ebay).

  Every five levels, there is an intermission.  The intermissions chronicle 
  the arrival of the four CS faculty in the MA/CS department.  

  There are six different level patterns, the seventh level is an identical 
  board to the first, and so on.
  
  Level 31 is the last level, when you complete it, you restart level 31 again.
  You will be treated to a random intermission each time that you complete
  level 31 (hey... it's not easy to get that far!  You deserve a reward!).

  It is possible to insert additional coins and continue from the spot that 
  you ran out of lives.  You have 9 seconds to insert additional coins and 
  press Player 1 start to do this.  It is useful if you are just short of the 
  next intermission.  

  This game contains dozens, if not 100 inside jokes to the department.  If 
  you really want to know what they all mean, come up for a visit.  You can
  find us in Sault Ste. Marie, MI!  

  This game has been successfully compiled on:
       Linux (using gcc)
       Windows (cygwin gcc)
       Windows (Dev-Cpp)
       Solaris (using gcc)
       Mac OS X (using gcc)
       
  If you successfully compile it for some other platform, please let us know!


  See the other SDL demonstrations at 
  http://furball.schemm.lssu.edu/SDL

 */





#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"


// Cygwin does not have random
#ifdef WIN32
#define RAND rand()
#endif
#ifndef WIN32
#define RAND random()
#endif

// Need to set up the different seed random functions too.
// Since demo_play needs a constant seed value, we can't do all 
// the function stuff here, all we can do is the function name.
#ifdef WIN32
#define SRAND  srand
#endif
#ifndef WIN32
#define SRAND  srandom
#endif



// Hold all the data for one student.  Going to make an array of these for
// all the students on the screen
struct STUDENT
  {
  int x;
  int y;
  int face;
  int active;
  };
typedef struct STUDENT Student;



// screen_flags is primarily a global variable so that we can switch between
// FullScreen and Windowed at will.
int screen_flags;
SDL_Surface * screen;

SDL_Surface * player;
SDL_Surface * brick;
SDL_Surface * book;
SDL_Surface * crimp;
SDL_Surface * cola;
SDL_Surface * cable;
SDL_Surface * fabbri;
SDL_Surface * student;
SDL_Surface * boger;
SDL_Surface * roof;
SDL_Surface * logo;
SDL_Surface * levelglyph;
SDL_Surface * BigBoger;
SDL_Surface * ibm_360;
SDL_Surface * rock;
SDL_Surface * crater;
SDL_Surface * dino1;
SDL_Surface * dino2;
SDL_Surface * dino3;
SDL_Surface * road;
SDL_Surface * signs;
SDL_Surface * house;
SDL_Surface * trucks;
SDL_Surface * misc;
SDL_Surface * BigTwig;
SDL_Surface * BigSchemm;
SDL_Surface * BigFabbri;
SDL_Surface * movers;
SDL_Surface * cats;
SDL_Surface * caset;
SDL_Surface * zombie;
SDL_Surface * dean;
SDL_Surface * pacitems;

SDL_Surface * littlefont;
SDL_Surface * bigfont;



#define WIDTH 600
#define HEIGHT 450
#define OFFSET 30

int ypos;    // The players y pos (will be rounded for screen work)
int yvel;    // The players y velocity
int xpos;       // The players current x position
int xface;      // Whether the player is facing left or right
int jump;       // Whether the player is jumping or not
int twigpower;  // Whether or not Twig has SuperJump



int fypos;   // Fabbri position
int fyvel;
int fxpos;
int fxface;

int fstatus;

Student students[20];  // Student data

int bogerx;            // Boger data
int bogery;
int bogerstat;
int bogerhouse;

int coins;           // Number of coins in reserve

#define GRAV 3.0     // Physics constants for TwigMan
#define JUMP -25.0
#define TERM 15


int p1score;         // Game data
int newlife;
int hiscore;
int cableleft;
int lives;
int level;

int tick;          // Used for system tick for animations.
int teaser_tick;

// Sound info.  Symbolic sound names make it slightly easier to read the 
// code.
Mix_Chunk * sounds[23];  
#define endgame 0
#define evil_laugh 1
#define appear 2
#define get_tester 3
#define money 4
#define can_open 5
#define steal 6
#define jumpping 7
#define squish 8
#define falling 9
#define death 10     
#define bookdrop 11  
#define get_book 12
#define drive 13
#define intermission 14
#define crash 15
#define thud 16
#define flower 17
#define meow 18
#define purr 19
#define gun 20
#define roar 21
#define life 22
#define NUM_CHANNELS 16

Mix_Music * musics[2];
#define bg 0
#define inter1 1

// We have to store the filepaths for the sound and config files in a 
// separate file, so that the automake system will make them 'to-spec'
// at build time.  Hardcoding paths may be a bad thing...  Although, 
// Frankly, I can't find any other examples of doing this 'the correct way'
// I'm sure someone on the Gnu team will see this and have a cow, but hey, 
// it works, it's portable, and that's what matters most.  Esp. when you
// don't provide good novice tutorials on how to do this stuff!!!!!

#include "files.h"
char SAVE_FILE[1000];



void do_sounds(Mix_Chunk* what);
void read_CONFIG();
void handle_escape();

int keys[10];           // Allow user defineable keys
#define JUMPKEY      0
#define LEFTKEY    1
#define RIGHTKEY   2
#define PAUSEKEY   3
#define COINKEY    4
#define STARTKEY   5
#define QUITKEY    6
#define FSKEY      7



// This is the background map.  I did it like this for two reasons.
//   1. I wanted to be able to change it without firing up the GIMP.
//   2. I wanted an easy way to check whether the player was standing on
//      a block or air.

char map[15][20];  // This holds the current map

// These are the master start maps for each level
char level1[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {2,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0},
		      {1,1,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0},
		      {0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,4},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
		      {0,4,0,0,0,0,0,0,1,1,1,4,0,0,0,0,0,0,0,0},
		      {1,1,0,0,0,1,0,0,0,0,0,1,1,0,0,0,0,1,1,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,1,1,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0},
		      {0,0,0,0,0,0,3,0,0,0,0,0,0,1,1,1,1,0,0,0},
		      {2,1,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0},
		      {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0},
		      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level2[15][20]= {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0},
                      {3,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1},
                      {0,0,0,1,1,1,1,1,1,0,2,0,0,0,0,3,0,0,0,0},
                      {0,0,0,4,0,0,0,0,0,0,1,1,0,0,0,1,0,0,0,0},
                      {0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0},
                      {0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0},
                      {3,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,1,1,1,0},
                      {1,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,4},
                      {0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
                      {0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,1,1,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level3[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,4,0,0,0,0,0,0,0,3,0,0,0,0,0,0},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0},
                      {1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,1,1,1,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {4,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,1,1,1,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level4[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,2,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0},
		      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1},
		      {0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,1,1,0,0},
		      {0,4,0,0,0,0,1,1,1,0,0,0,0,0,1,1,0,0,0,0},
		      {1,1,1,1,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,1,0,0,0,0,1,1,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0},
		      {0,3,0,0,1,0,0,1,1,0,0,0,0,0,1,1,1,1,1,1},
		      {0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,4,0},
		      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level5[15][20]= {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,3,0},
                      {0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,1,1,1,0},
                      {0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,2,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0},
                      {0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0},
                      {0,4,0,0,0,0,0,0,1,1,1,0,0,0,0,0,3,0,0,0},
                      {1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,3,0,0,0,0,1,1,1,0,0,0,0,4,0,0},
                      {0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level6[15][20] = {{3,0,0,0,0,0,0,0,0,3,3,0,0,0,0,0,0,0,0,3},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,4,0,4,0,4,0,4,0,4,4,0,4,0,4,0,4,0,4,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,2,2,0,0,0,0,0,0,0,0,0,0,0,0,2,2,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level7[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,4,0,0,0,4,0,0,0,0,0,0,0,0,4,0,0,0,4,0},
		      {1,1,0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0,1,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,3,0,0,3,0,0,0,0,0,0,0,0},
		      {1,1,0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0,1,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {1,1,0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0,1,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {3,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,0,0,3},
		      {1,1,0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0,1,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level8[15][20]= {{0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0},
                      {0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0},
                      {3,1,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,1,1,3},
                      {0,0,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,1},
                      {0,0,0,0,4,0,0,1,1,2,2,1,1,0,0,4,0,0,0,0},
                      {0,0,0,0,0,0,0,1,0,2,2,0,1,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,0,1,1,1,1,1,1,0,1,1,1,1,1,1},
                      {0,0,0,0,0,0,0,0,0,4,4,0,0,0,0,0,0,0,0,0},
                      {0,1,0,0,1,1,1,0,1,0,0,1,0,0,1,1,1,0,1,0},
                      {3,1,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,3},
                      {0,1,1,1,1,0,1,1,1,0,0,1,1,1,0,1,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level9[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,4,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0},
		      {1,1,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,1,1},
		      {0,0,0,0,0,0,1,0,0,0,0,0,0,1,1,1,0,4,0,0},
		      {0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,3,0,0,3,0,0,0,0,0,0,0,0},
		      {0,0,1,1,1,1,1,1,1,0,0,1,1,1,1,1,1,1,0,0},
		      {0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2},
		      {1,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
		      {0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
		      {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
		      {0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,2,0,0,0,0},
		      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level10[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,4,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0},
		      {0,1,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,1,0},
		      {0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0},
		      {1,0,1,0,1,0,1,0,1,0,0,1,0,1,0,1,0,1,0,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0},
		      {0,1,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,1,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2},
		      {1,0,1,0,1,0,1,0,1,0,0,1,0,1,0,1,0,1,0,1},
		      {0,2,0,0,0,4,0,0,0,0,0,0,2,0,0,0,0,0,0,0},
		      {0,1,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,1,0},
		      {0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0},
		      {1,0,1,0,1,0,1,0,1,0,0,1,0,1,0,1,0,1,0,1},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0},
		      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level11[15][20]= {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {4,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,3},
		      {1,1,1,1,1,1,0,0,0,3,0,0,0,0,1,1,1,1,1,1},
		      {0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,3,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4},
		      {1,1,1,1,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,1},
		      {0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0},
		      {1,1,1,1,1,1,0,0,0,3,0,0,0,0,1,1,1,1,1,1},
		      {0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		      {0,0,0,0,0,0,2,0,0,0,0,0,0,2,0,0,0,0,0,0},
		      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level12[15][20] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {3,0,0,0,0,0,0,0,4,0,0,4,0,0,0,0,0,0,0,3},
                      {1,0,1,1,0,1,1,0,1,0,0,1,0,1,1,0,1,1,0,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,0,0,0,0,0,0,0,0,4,4,0,0,0,0,0,0,0,0,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
                      {0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0},
                      {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,0,0,1},
                      {0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level13[15][20]= {{4,4,4,4,4,4,4,0,0,0,0,0,0,0,0,0,0,0,3,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,2,0,0,0,1,1,1,1,1,1,1,1,1,1,1},
                      {0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0},
                      {0,0,0,0,0,2,0,0,0,0,0,0,0,0,1,1,1,1,1,1},
                      {0,0,0,0,0,2,0,1,1,1,1,1,1,1,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,0,0,2,0,0,0,0,0,0,0,0,0,0},
                      {0,0,2,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,2,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1},
                      {0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0},
                      {0,0,0,2,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level14[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0},
                      {0,0,0,0,0,0,0,0,0,4,4,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,1,0,0,1,1,0,0,1,0,0,0,0,0,0},
                      {0,4,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,4,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0},
                      {0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,1,3,3,1,0,0,0,0,0,0,0,0},
                      {0,0,2,0,0,1,0,0,0,0,0,0,0,0,1,0,0,2,0,0},
                      {0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0},
                      {0,0,0,0,0,0,0,2,0,0,0,0,2,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level15[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0},
                      {0,0,0,4,0,0,0,0,0,0,1,1,0,0,0,0,4,0,0,0},
                      {0,0,0,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0},
                      {0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,0,0,0,3,0},
                      {0,0,0,0,0,0,1,0,0,0,0,2,0,0,0,0,0,1,1,1},
                      {0,2,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0},
                      {1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0},
                      {0,0,0,1,1,1,1,1,0,0,0,0,0,0,1,1,0,0,0,0},
                      {0,2,0,0,0,0,0,4,0,0,0,0,0,0,0,0,1,0,3,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level16[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,3,0,0,0,0,3,0,0,0,0,0,0,0,0,0,4,0,0},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
                      {2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0},
                      {1,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,1},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
                      {2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,1},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

char level17[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,1,1,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0},
                      {1,0,0,0,0,0,3,0,0,1,1,1,1,1,1,1,0,0,4,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,1,1,1,1,1,1,0,0,0,4,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,3,0,0,0,0,0,1,0,0,0,0,1,1,1,1,1,1,1,0},
                      {0,1,0,0,0,0,0,0,0,1,0,0,0,0,4,0,0,0,0,0},
                      {0,0,1,1,1,0,0,0,0,0,0,1,0,0,0,0,0,2,0,1},
                      {0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,1,1,1},
                      {0,0,0,0,0,0,1,1,1,1,1,0,0,1,0,3,0,0,0,0},
                      {0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };




char level18[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,0},
                      {0,1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,0},
                      {0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0},
                      {0,1,0,0,1,0,0,1,1,1,1,1,1,0,0,1,0,0,1,0},
                      {0,1,0,0,1,0,0,1,4,3,3,4,1,0,0,1,0,0,1,0},
                      {3,1,0,0,1,0,0,1,1,1,1,1,1,0,0,1,0,0,1,3},
                      {0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0},
                      {0,1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,0},
                      {0,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,0},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level19[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,0},
                      {0,0,0,1,1,3,0,0,0,0,0,0,0,0,0,0,1,1,0,0},
                      {0,0,0,4,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,3,1,1,0,0,0,0,0},
                      {0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0},
                      {0,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,1,1,0},
                      {0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0},
                      {3,1,1,0,4,0,0,0,0,0,0,0,0,0,0,1,1,3,0,0},
                      {0,0,0,0,0,0,1,1,0,0,0,0,4,0,0,0,0,0,0,0},
                      {0,0,0,1,1,0,0,0,0,0,1,1,0,0,0,0,0,0,1,1},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level20[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,4,4,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0},
                      {0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0},
                      {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                      {0,0,0,0,0,2,0,0,0,0,0,0,0,0,2,0,0,0,0,0},
                      {0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0},
                      {3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3},
                      {0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0},
                      {0,0,0,0,0,0,0,3,0,0,0,0,3,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0},
                      {0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };


char level21[15][20]={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {0,0,1,0,0,0,0,1,1,0,0,1,1,0,1,0,1,0,4,0},
                      {0,0,1,0,3,0,1,0,0,0,1,0,0,0,1,3,1,0,4,0},
                      {0,0,1,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,4,0},
                      {0,0,1,0,0,0,0,0,1,0,0,0,1,0,1,0,1,0,0,0},
                      {0,0,1,1,1,0,1,1,0,0,1,1,0,0,1,1,1,0,4,0},
                      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                      {1,1,0,0,1,0,1,0,1,0,0,0,1,1,1,0,0,1,1,0},
                      {1,0,1,0,1,3,1,0,1,0,3,0,1,0,0,0,1,0,0,0},
                      {1,1,0,0,1,0,1,0,1,0,0,0,1,1,0,0,0,1,0,0},
                      {1,1,0,0,1,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0},
                      {1,0,1,0,1,1,1,0,1,1,1,0,1,1,1,0,1,1,0,0},
                      {0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,0},
                      {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };







// This is used as we add new levels to save some typing!
#define NUM_LEVEL 21



// This is the matrix for the demo play mode.  The numbers correspond
// to keystrokes that TwigMan gets.  Demo_play is always run off the same
// Random seed, so the same set of actions yields the same play each time.
// This set of actions gives a fairly varied show of what the gameplay is
// like.  The demo_play mode lasts 18 seconds.
// Left = 1, Right = 2, Up = 4
char demo_play[360] = {0,0,0,4,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,2,
		      2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
		      2,6,6,6,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
		      2,2,2,2,2,2,2,2,2,2,2,2,2,6,6,6,6,6,2,2,
		      2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,6,6,2,2,2,
		      1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,1,1,
		      1,1,1,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,
		      4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
		      1,1,1,1,2,2,2,2,2,2,2,2,2,2,0,0,0,0,0,0,
		      1,1,0,0,0,0,0,0,0,0,2,2,2,2,2,2,2,2,2,2,
		      2,2,2,2,2,2,2,2,2,0,0,0,0,1,1,1,1,1,1,1,
		      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		      1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,
		      2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,
		      0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,
		      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,
		      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,
		      0,0,0,0,0,0,0,0,0,2,2,6,2,2,2,2,2,2,2,2};


// If you are using XPM's, you need to include the file as part of the code.
// Note that with most graphics programs the name of the array is similar to, 
// but not identical to the name of the XPM inside the file. 

#include "../images/NewerTwig3.xpm"
#include "../images/book.xpm"
#include "../images/fabbri.xpm"
#include "../images/cola.xpm"
#include "../images/cable.xpm"
#include "../images/man.xpm"
#include "../images/boger30.xpm"
#include "../images/roof.xpm"
//#include "../images/twigman_logo.xpm"
#include "../images/level.xpm"
#include "../images/grider.xpm"

// These are extra xpms for the intermissions.
#include "../inter_graphics/BigBoger2.xpm"
//#include "../inter_graphics/ibm360_2.xpm"
#include "../inter_graphics/rock.xpm"
#include "../inter_graphics/crater.xpm"
//#include "../inter_graphics/dino1.xpm"
#include "../inter_graphics/dino2.xpm"
#include "../inter_graphics/dino3.xpm"
//#include "../inter_graphics/road.xpm"
#include "../inter_graphics/signs.xpm"
#include "../inter_graphics/house.xpm"
#include "../inter_graphics/trucks.xpm"
#include "../inter_graphics/misc.xpm"
#include "../inter_graphics/BigTwig2.xpm"
#include "../inter_graphics/BigSchemm2.xpm"
#include "../inter_graphics/mover.xpm"
#include "../inter_graphics/cats.xpm"
#include "../inter_graphics/BigFabbri2.xpm"
//#include "../inter_graphics/caset.xpm"
//#include "../inter_graphics/zombie.xpm"
#include "../inter_graphics/dean.xpm"
#include "../inter_graphics/game_items.xpm"

#include "Fixed_print.h"

#define lfw 10
#define lfh 25
#define bfw 28
#define bfh 72

// xprint is a function for printing strings on the screen.  It is very 
// useful in conjunction with sprintf for printing anything on the screen!

void big_xprint(char* str, int x, int y)
  {
  SDL_Rect src;
  SDL_Rect dest;
  char* curr;

  src.w = bfw; src.h = bfh;
  dest.w = bfw; dest.h = bfh; dest.y = y; dest.x = x;

  curr = str;
  while (curr[0] != 0)
    {
    src.x = (curr[0] % 16) * bfw;
    src.y = (curr[0] / 16) * bfh;
    SDL_BlitSurface(bigfont, &src, screen, &dest);
    dest.x += bfw;
    curr++;
    }
  }



// xprint is a function for printing strings on the screen.  It is very 
// useful in conjunction with sprintf for printing anything on the screen!

void little_xprint(char* str, int x, int y)
  {
  SDL_Rect src;
  SDL_Rect dest;
  char* curr;

  src.w = lfw; src.h = lfh;
  dest.w = lfw; dest.h = lfh; dest.y = y; dest.x = x;

  curr = str;
  while (curr[0] != 0)
    {
    src.x = (curr[0] % 16) * lfw;
    src.y = (curr[0] / 16) * lfh;
    SDL_BlitSurface(littlefont, &src, screen, &dest);
    dest.x += lfw;
    curr++;
    }
  }






// This is a function that I wrote just after Twigs in Space.  It provides a
// program independent way to set the screen size. Note that screen_flags is 
// a global variable that can be changed before calling this function.  It 
// is quite possible to re-call SetVideoMode in the middle of your game to 
// resize the screen.  Of course, you lose everything that was on the old 
// screen when you do (you will need to completely redraw it).
void change_screen(int x, int y)
  {
  screen = SDL_SetVideoMode(x, y, 16, screen_flags);
  SDL_SetColorKey(screen,SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));
  if (screen == NULL)
    {
    fprintf(stderr, "Error changing screen mode: %s\n", SDL_GetError());
    exit(1);
    }
  }

// I got tired of this cluttering up main, so I made it a separate function.
// You will find versions of this in most of my SDL programs.  They vary 
// slightly depending on what other libraries I am using, and other 
// pedestrian inits I am doing.
void init_SDL_stuff()
  {
  int test;
  Uint16 format;

#if ICON_HACK==1
  setenv ("SDL_VIDEO_X11_WMCLASS", "twigman", 1);
#endif


  if((SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER)==-1))
    {
    printf("CHEESE\n");
    printf("Could not initialize SDL: %s.\n", SDL_GetError());
    exit(-1);
    }

  
  test = Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 32);
  
  if (test == -1)
    {
    printf("Mix_OpenAudio error\n");
    printf("Oh My Goodness, an error : %s", Mix_GetError());
    
    exit(1);
    }
  
  
  /* Clean up on exit */
  atexit(handle_escape);
  atexit(Mix_CloseAudio);
  atexit(SDL_Quit);
  
  
                   
  // Make the initial version of the main screen
  screen_flags = SDL_SWSURFACE | SDL_SRCCOLORKEY;
  read_CONFIG();
  change_screen(WIDTH, HEIGHT+OFFSET);
    
  SDL_WM_SetCaption("Twig Man", "Twig Man");
  SDL_ShowCursor(SDL_DISABLE);

    

  FixedPrint_Setup();

  // Channel setup
  Mix_AllocateChannels(NUM_CHANNELS);
  Mix_ReserveChannels(NUM_CHANNELS);
  Mix_GroupChannels(0,15,1); // All channels in one group!
  
  } 

// Collision detection used for Boger and Fabbri.  A simple circle-circle
// detection algorithm.  Got rid of the sqrt to speed things up.
int testcollide(int c1x, int c1y, int r1, int c2x, int c2y, int r2)
  {
  return  ( ((c2x - c1x) * (c2x - c1x)) +
  ((c2y - c1y) * (c2y - c1y)) )   < 
  ((r1 + r2) * (r1 + r2)) ;
  }


/****************************************************************************
 *
 * do_sounds - play a sound out of the LRU channel.  Makes the addition of
 * playing a sound a one liner from anywhere else in the code.  The function
 * will find an unused channel if one is available, otherwise it will pick
 * the oldest one.
 ****************************************************************************/
void do_sounds(Mix_Chunk* what)
  {
  int channel;
  
  channel = Mix_GroupAvailable(1);
  if (channel == -1)
    channel = Mix_GroupOldest(1);
  Mix_PlayChannel(channel, what, 0);
  }

/***************************************************************************
 *
 * read_CONFIG - Read in a config file (if it doesn't exist, use defaults)
 *   to set which keys are used for which operations.  It also lets the user
 *   determine whether to start in full screen or windowed mode.
 *
 ****************************************************************************/
void read_CONFIG()
  {
  FILE * infile;
  char array[800];

  // Set defaults
  keys[JUMPKEY] = SDLK_UP;
  keys[LEFTKEY] = SDLK_LEFT;
  keys[RIGHTKEY] = SDLK_RIGHT;
  keys[PAUSEKEY] = SDLK_p;
  keys[COINKEY] = SDLK_5;
  keys[STARTKEY] = SDLK_1;
  keys[QUITKEY] = SDLK_ESCAPE;
  keys[FSKEY] = SDLK_F11;

  screen_flags = SDL_SWSURFACE | SDL_SRCCOLORKEY; // Windowed

  infile = fopen(CONF_FILE, "r");

  // If there is no config file, use default settings
  if (infile == NULL) return;

  // Otherwise, read file
  while (fscanf(infile, "%80s", array)!= EOF)
    {

    while(array[0] == '#')  // Get rid of comment lines
      {
      if (fscanf(infile, "%80s", array)== EOF)
	{
	fclose(infile);
	return;
	}
      }

    // Got a mode line
    if (strcmp(array, "mode") == 0)
      {
      fscanf(infile, "%80s", array);
      if (strcmp(array, "FULLSCREEN") == 0)
        screen_flags = SDL_SWSURFACE | SDL_FULLSCREEN|SDL_SRCCOLORKEY;
      else
        screen_flags = SDL_SWSURFACE| SDL_SRCCOLORKEY;
      }

    // Got an upkey line
    if (strcmp(array, "jumpkey") == 0) fscanf(infile, "%d",&keys[JUMPKEY]);
    
    // Got a leftkey line
    if (strcmp(array, "leftkey") == 0) fscanf(infile, "%d",&keys[LEFTKEY]);

    // Got a rightkey line
    if (strcmp(array, "rightkey") == 0) fscanf(infile, "%d",&keys[RIGHTKEY]);

    // Got a pausekey line
    if (strcmp(array, "pausekey") == 0) fscanf(infile, "%d",&keys[PAUSEKEY]);

    // Got a coinkey line
    if (strcmp(array, "coinkey") == 0) fscanf(infile, "%d",&keys[COINKEY]);

    // Got a startkey line
    if (strcmp(array, "startkey") == 0) fscanf(infile, "%d",&keys[STARTKEY]);

    // Got a quitkey line
    if (strcmp(array, "quitkey") == 0) fscanf(infile, "%d",&keys[QUITKEY]);

    // Got a fskey line
    if (strcmp(array, "fskey") == 0) fscanf(infile, "%d",&keys[FSKEY]);

    } // End while

  fclose(infile);
  return;
  }




#include "intermission.h"

// Code to actually draw the screen.  I usually put this in a separate function
// so that it does not clutter up main.
int draw_screen(int real)
  {
  SDL_Rect rect;
  SDL_Rect pos;
  
  int x, y;
  int loop;

  char str[100];
  SDL_Color c;
  
  // Not the worlds most efficient screen drawing method, but it's fast 
  // enough.  

  // Start by blanking the screen
  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0));

  // Then draw the griders, books, colas, and testers
  rect.y = 0; rect.w = 30; rect.h = 30;
  for (y = 0; y < 15; y++)
    {
    for (x = 0; x < 20; x++)
      {
      rect.x = 0;
      if (map[y][x] == 1)
	{
	pos.x = x * 30; pos.y = y * 30 + OFFSET;
	SDL_BlitSurface(brick, &rect, screen, &pos);
	}
      if (map[y][x] == 2)
	{
	pos.x = x*30; pos.y = y*30 + OFFSET;
	SDL_BlitSurface(book, &rect, screen, &pos);
	}
      if (map[y][x] == 3)
	{
	rect.x = (tick)%6 * 30;
	pos.x = x*30; pos.y = y*30 + OFFSET;
	SDL_BlitSurface(cola, &rect, screen, &pos);
	}
      if (map[y][x] == 4)
	{
	rect.x = (tick/10)%6 * 30;
	pos.x = x*30; pos.y = y*30 + OFFSET;
	SDL_BlitSurface(cable, &rect, screen, &pos);
	}
      }
    }
  // Now draw Fabbri, if he is on the screen.
  if (fstatus != 0)
    {
    rect.x = 30 * fxface; rect.y = 0; rect.w = 30; rect.h = 30;
    pos.x = fxpos; pos.y = (int)fypos+.5 + OFFSET;
    SDL_BlitSurface(fabbri, &rect, screen, &pos);
    }

  // Now draw any students
  for (loop = 0; loop < 20; loop++)
    {
    if (students[loop].active == 1)
      {
      rect.x = 20 * (tick%8); rect.y = 20 * students[loop].face; 
      rect.w = 20; rect.h = 20;
      pos.x = students[loop].x ; pos.y = students[loop].y + OFFSET;
      SDL_BlitSurface(student, &rect, screen, &pos);
      }
    }
  

  // TwigPower bar
  rect.x = 0; rect.y = 19; rect.w = twigpower* 3; rect.h = 11;
  SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 65,105,225));
  sprintf(str, "TwigPower TwigPower TwigPower TwigPower TwigPower TwigPower TwigPower TwigPower TwigPower TwigPower");
  c.r = 0; c.g = 0; c.b = 0; 
  FixedPrint(screen, 0, 20, str, &c);

  // Print boger
  if (bogerstat > 0)
    {
    rect.x = 0; rect.y = 0; rect.w = 25; rect.h = 30;
    pos.x = bogerx; pos.y = bogery + OFFSET;
    SDL_BlitSurface(boger, &rect, screen, &pos);

    rect.x = 0; rect.y = 0; rect.w = 100; rect.h = 32;
    pos.x = bogerhouse; pos.y = 0;
    SDL_BlitSurface(roof, &rect, screen, &pos);
    }

  // Need to show the level
  if (level & 1)
    {
    rect.x = 0; rect.y = 0; rect.w = 20; rect.h = 20;
    pos.x = 570; pos.y = 425 + OFFSET + 1;
    SDL_BlitSurface(levelglyph, &rect, screen, &pos);
    }
  if (level & 2)
    {
    rect.x = 20; rect.y = 0; rect.w = 20; rect.h = 20;
    pos.x = 540; pos.y = 425 + OFFSET + 1;
    SDL_BlitSurface(levelglyph, &rect, screen, &pos);
    }
  if (level & 4)
    {
    rect.x = 40; rect.y = 0; rect.w = 20; rect.h = 20;
    pos.x = 510; pos.y = 425 + OFFSET + 1;
    SDL_BlitSurface(levelglyph, &rect, screen, &pos);
    }
  if (level & 8)
    {
    rect.x = 60; rect.y = 0; rect.w = 20; rect.h = 20;
    pos.x = 480; pos.y = 425 + OFFSET + 1;
    SDL_BlitSurface(levelglyph, &rect, screen, &pos);
    }
  if (level & 16)
    {
    rect.x = 80; rect.y = 0; rect.w = 20; rect.h = 20;
    pos.x = 450; pos.y = 425 + OFFSET + 1;
    SDL_BlitSurface(levelglyph, &rect, screen, &pos);
    }
  
  if (level & 32)
    {
    rect.x = 100; rect.y = 0; rect.w = 20; rect.h = 20;
    pos.x = 420; pos.y = 425 + OFFSET + 1;
    SDL_BlitSurface(levelglyph, &rect, screen, &pos);
    }


  // Now the player
  // rect is the part of the player image that we are going to use.
  // Since the Surface has both a right facing and left facing image, 
  // We use rect.x to specify which face to show.  We can do more complicated
  // things with this also.  See: tilemaps.
  // Pos is the location on the screen where it is going to go.  Note: ypos
  // is a float (because of the gravity math), so we need to round then 
  // convert to an int.
  rect.x = 30 * xface; rect.y = 0; rect.w = 30; rect.h = 30;
  pos.x = xpos; pos.y = (int)ypos+.5 + OFFSET;
	    
  SDL_BlitSurface(player, &rect, screen, &pos);

  // The score
  sprintf(str, "P1: %06d", p1score);
  little_xprint(str, 1, 0);

  sprintf(str, "High Score: %06d", hiscore);
  little_xprint(str, WIDTH/3, 0);

  sprintf(str, "Left: %02d", cableleft);
  little_xprint(str, 2*WIDTH/3+50, 0);

  // Need to show the lives remaining
  if (lives > 1)
    {
    rect.x = 30*xface ; rect.y = 0; rect.w = 30; rect.h = 30;
    pos.x = 5; pos.y = 420 + OFFSET + 1;
    
    SDL_BlitSurface(player, &rect, screen, &pos);
    }
  if (lives > 2)
    {
    rect.x = 30*xface ; rect.y = 0; rect.w = 30; rect.h = 30;
    pos.x = 40; pos.y = 420 + OFFSET + 1;
    
    SDL_BlitSurface(player, &rect, screen, &pos);
    }
  if (lives > 3)
    {
    rect.x = 30*xface ; rect.y = 0; rect.w = 30; rect.h = 30;
    pos.x = 75; pos.y = 420 + OFFSET + 1;
    
    SDL_BlitSurface(player, &rect, screen, &pos);
    }
  if (lives > 4)
    {
    rect.x = 30*xface ; rect.y = 0; rect.w = 30; rect.h = 30;
    pos.x = 110; pos.y = 420 + OFFSET + 1;
    
    SDL_BlitSurface(player, &rect, screen, &pos);
    }


  // If we are doing the teaser demo
  if ((real == 0)&&((tick/10)%2))
    {
    sprintf(str, "Game Over");
    little_xprint(str, 255, 210);
    }

  // Don't forget me.
  // Bonus question for those who actually looked at the code:
  // Why don't we do this Update for each SDL_BlitSurface?  Why only at the
  // end of the frame?  Hint: Copy this to the for loop above and you will 
  // immediately see why (and it has nothing to do with the speed of the 
  // machine that you are on.
  if (real != 2)
    SDL_UpdateRect(screen, 0,0,0,0);
	       
  }


/**************************************************************************
 * 
 * Death!
 * 
 **************************************************************************/
void do_death()
  {
  int loop;
  SDL_Event event;

  Mix_HaltMusic();
  do_sounds(sounds[death]);
  for (loop = 0; loop < 180; loop++)
    {
    draw_screen(1);
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    tick = (tick+1)%2520;
    }

  } // End death subroutine

/*****************************************************************************
 * 
 * End game delay loop - Play end of game music and show game over
 *
 *****************************************************************************/
void end_game()
  {
  int loop;
  SDL_Event event;

  for (loop = 0; loop < 100; loop++)
    {
    big_xprint("Game Over", 175, 200);
    SDL_UpdateRect(screen,0,0,0,0);
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    }

  }

/**************************************************************************
 * 
 * End of Game?
 * 
 **************************************************************************/
int do_end_of_game()
  {
  int loop;
  SDL_Event event;
  char str[100];
  int count;
  int mode;
  
  // Endgame!
  do_sounds(sounds[endgame]);
  end_game();
  
  mode = (coins > 0);// Hey look! I actually found a legit use for this C code
                    // oddity.  Woohoo! C Rules! Take that PASCAL!

  for (loop = 0; loop < 200; loop++)
    {
    count = 9 - (loop / 20);
    draw_screen(2);
    big_xprint("Continue?", 170, 100);
    if (mode == 0)
      {
      big_xprint("Insert Coin", 140, 200);
      sprintf(str, "%02d", count);
      big_xprint(str, 275, 300);
      }
    else
      {
      big_xprint("Press P1 Start", 105, 200);
      sprintf(str, "%02d", count);
      big_xprint(str, 275, 300);
      }
    SDL_UpdateRect(screen, 0,0,0,0);

    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    mode = 1;
	    loop = 0;
	    }
	  if ((event.key.keysym.sym == keys[STARTKEY])&&(coins > 0))
	    {
	    coins--;
	    lives = 3;
	    p1score = 0;
        newlife = 1;
	    return 1;  // The player is going to keep going!
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    tick = (tick+1)%2520;
    }
  return 0;
  } // End death subroutine

/**************************************************************************
 * 
 * Do intermission - Called at the successful completion of each level
 * 
 **************************************************************************/
void do_intermission(int level)
  {
  int loop;
  SDL_Event event;
  int ltemp;

  // Need an intermission sound!
  do_sounds(sounds[intermission]);
  Mix_HaltMusic();
  for (loop = 0; loop < 120; loop++)
    {
    draw_screen(1);
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    tick = (tick+1)%2520;
    } // End delay for end of screen

  if ((level % 5 != 0)&&(level!=63)) return;

  if (level <= 60) ltemp = level/5;  else ltemp = RAND % 12;

  if (ltemp==1)
    {
    show_intermission1();
    }
  if (ltemp == 2)
    {
    show_intermission2();
    }
  if (ltemp == 3)
    {
    show_intermission3();
    }
  if (ltemp == 4)
    {
    show_intermission4();
    }
  if (ltemp == 5)
    {
    show_intermission5();
    }
  if (ltemp == 6)
    {
    show_intermission6();
    }
if (ltemp == 7)
    {
    show_intermission7();
    }
  if (ltemp == 8)
    {
    show_intermission8();
    }
  if (ltemp == 9)
    {
    show_intermission9();
    }
  if (ltemp == 10)
    {
    show_intermission10();
    }
  if (ltemp == 11)
    {
    show_intermission11();
    }
  if (ltemp == 12)
    {
    show_intermission12();
    }

  } // End intermission


/****************************************************************************
 * 
 * pause_mode - Handle being in pause mode!  We wait here until they hit
 *              'P' again!
 * 
 ****************************************************************************/
void pause_mode()
  {
  int loop;
  SDL_Event event;
  SDL_Rect rect;
  SDL_Rect pos;
  
  while (1)
    {
    draw_screen(1);
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    return;
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	  if (event.key.keysym.sym == keys[PAUSEKEY])
	    {
	    return;
	    }
	}
      }
    tick = (tick+1)%2520;
    SDL_Delay(50);
    }

  }

/****************************************************************************
 *
 * read_save - Read a savefile and restore state (set global vars).
 * 
 ****************************************************************************/
int read_save()
  {
  FILE* infile;
  unsigned char a;
  unsigned char b;
  unsigned char c;
  int outer;
  int inner;

  // Open file
  infile = fopen(SAVE_FILE, "r");
  if (infile == NULL) return -1;  // No savefile, normal start

  // Fill in data
  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore

  ypos = (a<<2) + ((b&0xc0) >> 6)   - 45;
  yvel = (b&0x3f)  - 25;

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  
  xpos = (a<<2) + ((b&0xc0) >> 6);
  xface = (b & 0x20)>> 5;
  jump = (b & 0x10)>> 4;
  cableleft = (b & 0x0f);

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  twigpower = a;
  
  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  
  fypos = (a << 1) + ((b&0x80) >> 7);
  coins = (b & 0x7f) ;

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  fxpos = (a << 2) + ((b&0xc0) >> 6);
  level = (b & 0x3f) ;

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  bogerx = (a << 2) + ((b&0xc0) >> 6);
  lives = (b & 0x3c) >> 2;
  bogerstat = (b & 0x03);

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&c, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  tick = (a << 4) + ((b & 0xf0) >> 4);
  teaser_tick = ((b & 0x0f) << 8) + c;

  for (outer = 0; outer < 20; outer++)
    {
    if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
    if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
    if (fread(&c, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
    students[outer].x = (a << 4) + ((b & 0xc0) >> 4);
    students[outer].y = ((b & 0x3f) << 3) + ((c & 0xe0)>>5);
    students[outer].face = ((c & 0x10)>>4);
    students[outer].active = ((c & 0x08) >> 3);
    }

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&c, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  bogerx = (a << 2) + ((b & 0xc0) >> 6);
  bogerhouse = ((b & 0x03) << 8) + c   - 27;

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  bogery = (a << 1) + ((b&0x80) >> 7)   - 28;
  newlife = (b & 0x7f) ;
  
  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&c, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  p1score = (a << 12) + (b << 4) + ((c & 0xf0) >> 4);
  fyvel = ((c & 0x08) >> 3) * 5;
  fxface = ((c & 0x04) >> 2);
  fstatus = ((c & 0x02) >> 1);

  if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&b, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
  if (fread(&c, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
 
  hiscore = (a << 12) + (b << 4) + ((c & 0xf0) >> 4);

  for (outer = 0; outer < 15; outer++)
    {
    for (inner =0; inner < 20; inner++)
      {
      if (fread(&a, 1, 1, infile) != 1) return -1;  // Bad savefile, ignore
      map[outer][inner] = a;
      }
    }

  fclose (infile);

  // Need to delete savefile 
  remove(SAVE_FILE);

  // Wait for the user to resume play.
  pause_mode();
  return 3;
  }

/****************************************************************************
 * 
 * write_save - Write the current save state to a file so that we can resume
 *              at a later point.
 *
 ****************************************************************************/
void write_save()
  {
  FILE* outfile;

  unsigned char a;
  unsigned char b;
  unsigned char c;

  int outer;
  int inner;

  outfile = fopen(SAVE_FILE, "w");

  a = 0; a = (unsigned char)(((ypos+45) & 0x03fc) >> 2);
  b = 0; b = (unsigned char)((((ypos+45) & 0x03) << 6) + ((yvel+25) & 0x3f));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);

  a = 0; a = (unsigned char)((xpos & 0x03fc) >> 2);
  b = 0; b = (unsigned char)(((xpos & 0x03) << 6) + ((xface & 0x01) << 5) +
			     ((jump & 0x01) << 4) + (cableleft & 0x0f));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);

  a = 0; a = (unsigned char)(twigpower & 0xff);
  fwrite(&a, 1, 1, outfile);

  a = 0; a = (unsigned char)((fypos & 0x01fe) >> 1);
  b = 0; b = (unsigned char)(((fypos & 0x01) << 7) + (coins & 0x7f));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);

  a = 0; a = (unsigned char)((fxpos & 0x03fc) >> 2);
  b = 0; b = (unsigned char)(((fxpos & 0x03) << 6) + (level & 0x3f));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);

  a = 0; a = (unsigned char)((bogerx & 0x03fc) >> 2);
  b = 0; b = (unsigned char)(((bogerx & 0x03) << 6) + ((lives & 0x0f) << 2) +
			     (bogerstat & 0x03));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);

  a = 0; a = (unsigned char)((tick & 0x0ff0) >> 4);
  b = 0; b = (unsigned char)(((tick & 0x0f) << 4)+((teaser_tick & 0x0f00)>>8));
  c = 0; c = (unsigned char)((teaser_tick & 0x00ff));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);
  fwrite(&c, 1, 1, outfile);

  for (outer = 0; outer < 20; outer++)
    {
    a = 0; a = (unsigned char)((students[outer].x & 0x03fc) >> 2);
    b = 0; b = (unsigned char)(((students[outer].x & 0x03) << 6) + 
			((students[outer].y & 0x01f8) >> 3));
    c = 0; c = (unsigned char)(((students[outer].y & 0x07) << 5) + 
			       ((students[outer].face & 0x01) << 4)+
			       ((students[outer].active & 0x01) << 3));
    fwrite(&a, 1, 1, outfile);
    fwrite(&b, 1, 1, outfile);
    fwrite(&c, 1, 1, outfile);
    }

  a = 0; a = (unsigned char)((bogerx & 0x03fc) >> 2);
  b = 0; b = (unsigned char)(((bogerx & 0x03) << 6) +((bogerhouse&0x0300)>>8));
  c = 0; c = (unsigned char)((bogerhouse & 0x00ff));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);
  fwrite(&c, 1, 1, outfile);

  a = 0; a = (unsigned char)(((bogery+28) & 0x01fe) >> 1);
  b = 0; b = (unsigned char)((((bogery+28) & 0x01) << 7) + (newlife & 0x7f));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);

  a = 0; a = (unsigned char)((p1score & 0x000ff000) >> 12);
  b = 0; b = (unsigned char)((p1score & 0x00000ff0) >> 4);
  c = 0; c = (unsigned char)(((p1score & 0x0f)<< 4) + (((fyvel/5)&0x01)<<3) +
			     ((fxface & 0x01) << 2)+((fstatus & 0x01) << 1));
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);
  fwrite(&c, 1, 1, outfile);

  a = 0; a = (unsigned char)((hiscore & 0x000ff000) >> 12);
  b = 0; b = (unsigned char)((hiscore & 0x00000ff0) >> 4);
  c = 0; c = (unsigned char)((hiscore & 0x0000000f) << 4);
  fwrite(&a, 1, 1, outfile);
  fwrite(&b, 1, 1, outfile);
  fwrite(&c, 1, 1, outfile);

  for (outer = 0; outer < 15; outer++)
    {
    for (inner = 0; inner < 20; inner++)
      {
      a = 0; a = (unsigned char)map[outer][inner];
      fwrite(&a, 1, 1, outfile);
      }
    }

  fclose(outfile);
  }

void handle_escape()
  {
  if (lives > 0) write_save();
  }


void restart_level()
  {
  int loop;
  jump = 0;
  xpos = 230;
  ypos = 390;
  yvel = 0;
  xface = 0;
  
  fstatus = 0;   
  fyvel = 0;
  fypos = 0;
  fxface = 0;
  fxpos = 0;
  
  for (loop = 0; loop < 20; loop++)
    {
    students[loop].active = 0;
    }
  
  bogerstat = 0;
  bogerx = 0;
  bogery = 0;
  
  }


/****************************************************************************
 *
 * play_game
 *
 ****************************************************************************/
void play_game(int real)
  {

  SDL_Event event;
  Uint8 *keystate;
  SDL_Surface* temp;

  int x, y;                 // Temp variables for adding new item, 
                            // and map copy
  int loop;
  int newone;               // I worked out the logic for the demo before I 
                            // made some tweaks to the item drop.  The item
                            // drop affects the random number usage, so we 
                            // have to go through this complicated process to 
                            // restore it for the demo (since coming up with
                            // a new demo sequence would be a pain in the 
                            // behind.  This variable is used to get the 
                            // Random chance of a item drop, since it is needed
                            // in two spots.



  // Variable initialization is slightly different depending on if this 
  // is a real game, or the demo loop.
  if (real > 0)
    {
    SRAND (time(NULL));
    Mix_PlayMusic(musics[bg], -1);
    }
  else
    {
    SRAND (0);
    lives = 0;
    teaser_tick = 0;
    // Need to UnRandomize
    }

// If this was a restart from save file, we need to read in the save data,
// and enter pause mode at this point.  When we leave pause_mode, the main
// game loop starts again.

  if (real == 0)
    {
    // initial game start
    restart_level();
    // Initialize the current map with this level.
    for (x = 0; x < 20; x++)
      {
      for (y = 0; y < 15; y++)
	{
	map[y][x] = level1[y][x];
	}
      }
    level = 1;
    cableleft = 10;
    }
    
  
  if (real == 1)  // 
    {
    // initial game start
    restart_level();
    // New life every 15,000 points.  This keeps track of which one we 
    // are going to award next.  That way we can do a > check 
    // (instead of ==) and not keep awarding the same extra life 
    // multiple times.
    newlife = 1;

    // Initialize the current map with this level.
    for (x = 0; x < 20; x++)
      {
      for (y = 0; y < 15; y++)
	{
	map[y][x] = level1[y][x];
	}
      }
    //#################################
    // WORKING HERE CHANGE ME BACK!!!!!
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    level = 1;
    cableleft = 10;
    lives = 3;
    p1score = 0;
    }

  if ((real == 2) &&(read_save() == -1))
    {
    Mix_HaltMusic();
    return;
    }

  if (real == 2) 
    {
    if (cableleft == 0)
      {
      level++;
      if (level > 63) level = 63;
      restart_level();
      cableleft = 10;
      // Load a new map
      for (x = 0; x < 20; x++)
	{
	for (y = 0; y < 15; y++)
	  {
	  if (level % NUM_LEVEL == 0)	   map[y][x] = level21[y][x];
	  if (level % NUM_LEVEL == 1)   map[y][x] = level1[y][x];
	  if (level % NUM_LEVEL == 2)   map[y][x] = level2[y][x];
	  if (level % NUM_LEVEL == 3)   map[y][x] = level3[y][x];
	  if (level % NUM_LEVEL == 4)   map[y][x] = level4[y][x];
	  if (level % NUM_LEVEL == 5)   map[y][x] = level5[y][x];
	  if (level % NUM_LEVEL == 6)   map[y][x] = level6[y][x];
	  if (level % NUM_LEVEL == 7)   map[y][x] = level7[y][x];
	  if (level % NUM_LEVEL == 8)   map[y][x] = level8[y][x];
	  if (level % NUM_LEVEL == 9)   map[y][x] = level9[y][x];
	  if (level % NUM_LEVEL == 10)   map[y][x] = level10[y][x];
	  if (level % NUM_LEVEL == 11)   map[y][x] = level11[y][x];
	  if (level % NUM_LEVEL == 12)   map[y][x] = level12[y][x];
	  if (level % NUM_LEVEL == 13)   map[y][x] = level13[y][x];
	  if (level % NUM_LEVEL == 14)   map[y][x] = level14[y][x];
	  if (level % NUM_LEVEL == 15)   map[y][x] = level15[y][x];
	  if (level % NUM_LEVEL == 16)   map[y][x] = level16[y][x];
	  if (level % NUM_LEVEL == 17)   map[y][x] = level17[y][x];
	  if (level % NUM_LEVEL == 18)   map[y][x] = level18[y][x];
	  if (level % NUM_LEVEL == 19)   map[y][x] = level19[y][x];
	  if (level % NUM_LEVEL == 20)   map[y][x] = level20[y][x];
	  }
	}
      }
    real = 1;  // Convert from pre-load to real game play mode
    }
// *************************************************************************
// *
// * MAIN LOOP START 
// *
// *************************************************************************

 
  while (1)
    {
    if (ypos < -45) ypos = -45; // Can't jump off the top of the screen

    // Handle TwigMan jump
    if (jump == 1)
      {
       yvel += GRAV;
       if (yvel > TERM) yvel = TERM;
      ypos += yvel;

      if (ypos >= 390)
	{
	ypos = 390;
	jump = 0;
	}
      }

    if (yvel > 0) // Decending jump and land
      {
      if ((map[(int)((ypos+45)/30)][(xpos+15)/30] == 1) ||
	  (map[(int)((ypos+45)/30)][(xpos+14)/30] == 1))
	{
	yvel = 0; 
        jump = 0;
	ypos = 30 * (int)((ypos+15) / 30);
	}
      }
    
    // Falling
      if (((map[(int)((ypos+30)/30)][(xpos+15)/30] != 1) &&
	   (map[(int)((ypos+30)/30)][(xpos+14)/30] != 1))
	  &&(jump == 0))
      {
      jump = 1;
      yvel = 1;
      }
    



    /***********************************************************************/
    /*                                                                     */
    /* START OBJECT COLLECTION                                             */
    /*                                                                     */
    /***********************************************************************/

    // Book collection
    if ((ypos>0)&&(map[(int)((ypos)/30)][(xpos+15)/30] == 2))
      {
      map[(int)((ypos)/30)][(xpos+15)/30] = 0;
      if (real == 1)
	p1score += 10;
      do_sounds(sounds[get_book]);
      }
    // Cola Collection
    if ((ypos>0)&&((map[(int)((ypos)/30)][(xpos+15)/30] == 3)))
      {
      map[(int)((ypos)/30)][(xpos+15)/30] = 0;
      if (real == 1)
	p1score += 100;
      twigpower = 200;
      do_sounds(sounds[can_open]);
      }
    // Cable Tester Collection
    if ((ypos>0)&&(map[(int)((ypos)/30)][(xpos+15)/30] == 4))
      {
      map[(int)((ypos)/30)][(xpos+15)/30] = 0;
      if (real == 1)
	p1score += 300;
      cableleft -= 1;
      do_sounds(sounds[get_tester]);
      if (cableleft == 0)  // They completed this level!
	{
	do_intermission(level);
	level++;
	Mix_PlayMusic(musics[bg], -1);
	// Need to reset positions for the next level!
	if (level > 63) level = 63;
	restart_level();
	cableleft = 10;
	// Load a new map
	for (x = 0; x < 20; x++)
	  {
	  for (y = 0; y < 15; y++)
	    {
	    if (level % NUM_LEVEL == 0)	   map[y][x] = level21[y][x];
	    if (level % NUM_LEVEL == 1)   map[y][x] = level1[y][x];
	    if (level % NUM_LEVEL == 2)   map[y][x] = level2[y][x];
	    if (level % NUM_LEVEL == 3)   map[y][x] = level3[y][x];
	    if (level % NUM_LEVEL == 4)   map[y][x] = level4[y][x];
	    if (level % NUM_LEVEL == 5)   map[y][x] = level5[y][x];
	    if (level % NUM_LEVEL == 6)   map[y][x] = level6[y][x];
	    if (level % NUM_LEVEL == 7)   map[y][x] = level7[y][x];
	    if (level % NUM_LEVEL == 8)   map[y][x] = level8[y][x];
	    if (level % NUM_LEVEL == 9)   map[y][x] = level9[y][x];
	    if (level % NUM_LEVEL == 10)   map[y][x] = level10[y][x];
	    if (level % NUM_LEVEL == 11)   map[y][x] = level11[y][x];
	    if (level % NUM_LEVEL == 12)   map[y][x] = level12[y][x];
	    if (level % NUM_LEVEL == 13)   map[y][x] = level13[y][x];
	    if (level % NUM_LEVEL == 14)   map[y][x] = level14[y][x];
	    if (level % NUM_LEVEL == 15)   map[y][x] = level15[y][x];
	    if (level % NUM_LEVEL == 16)   map[y][x] = level16[y][x];
	    if (level % NUM_LEVEL == 17)   map[y][x] = level17[y][x];
	    if (level % NUM_LEVEL == 18)   map[y][x] = level18[y][x];
	    if (level % NUM_LEVEL == 19)   map[y][x] = level19[y][x];
	    if (level % NUM_LEVEL == 20)   map[y][x] = level20[y][x];
	    }
	  }

	} // End of done with level - init new
      } // End if cable tester

    // Add a new cola or cable tester
    newone = RAND % 200;
    // The if test is ugly because I added the tick%180 after working out the
    // demo sequence.  With that addition, the random number use changes, so 
    // you don't get the same game as before.  It took too long to generate an
    // interesting sequence last time, and I didn't want to go through it again
    // so I kludged in this variable and the ugly if statement to make the demo
    // play the same as before.  
    if ((((newone < 1)||((tick%240)==0))&&(real==1)) ||
	((newone < 1) && (real == 0)))
      // Max 9 secs between drops! But only for real play!
      {
      x = RAND % 20;
      y = RAND % 12 + 1;
      if (RAND % 2 < 1)
	{
	// Add a new tester
	if (map[y][x] == 0) 
	  {
	  map[y][x] = 4;
	  do_sounds(sounds[appear]);
	  }
	}
      else
	{
	// Add a new cola
	if (map[y][x] == 0) 
	  {
	  map[y][x] = 3;
	  do_sounds(sounds[appear]);
	  }
	}
      } // End add new cola or cable tester
    /***********************************************************************/
    /*                                                                     */
    /*  END OBJECT COLLECTION                                              */
    /*                                                                     */
    /***********************************************************************/



    /***********************************************************************/
    /*                                                                     */
    /*  START FABBRI                                                       */
    /*                                                                     */
    /***********************************************************************/

    // A new Fabbri?
    if ((fstatus == 0) && (RAND % 100) < 1)
      {
	if ((RAND % 2) == 0)
	  {
	    fstatus = 1;
	    fxface = 0;
	    fxpos = WIDTH-30;
	    fyvel = 0;
	    fypos = RAND % 13;
	    while (map[fypos][19] == 1)   fypos = RAND % 13;
	    fypos =fypos * 30 + OFFSET;
	    do_sounds(sounds[drive]);
	  }
	else
	  {
	    fstatus = 1;
	    fxface = 1;
	    fxpos = 0;
	    fyvel = 0;
	    fypos = RAND % 13;
	    while (map[fypos][0] == 1)   fypos = RAND % 13;
	    fypos = fypos * 30 + OFFSET;
	    do_sounds(sounds[drive]);
	  }
      }
    
    if (fstatus == 1)  //  Update Fabbri if he exists!
      {
      if ((twigpower==0)&&
	  (testcollide(xpos+15, ypos+15, 14, fxpos+15, fypos+15, 15)))
	{
	do_death();
	// Need to reset for level restart
	lives--;
	if (lives == 0) 
	  {
	  if (do_end_of_game() == 0)	  return;
	  }
	Mix_PlayMusic(musics[bg], -1);
	restart_level();
	} // End death by Fabbri

      // Fabbri falling
      if (map[(int)((fypos+30)/30)][(fxpos+15)/30]!=1)
	{
       	fyvel = 5;
	}

      // Fabbri lands!
      if (map[(int)((fypos+30)/30)][(fxpos+15)/30] == 1)
	{
	fyvel = 0;
	fypos = 30 * (int)((fypos) / 30);
	}

      // Fabbri steals the books
      if (map[(int)((fypos+10)/30)][(fxpos+15)/30] == 2)
	{
	map[(int)((fypos+10)/30)][(fxpos+15)/30] = 0;
	do_sounds(sounds[steal]);
	}

      // Update his position
      fypos += fyvel;
      if (fxface == 0) fxpos -= 7; else fxpos += 7;
      
      // If he is off the screen
      if ((fxpos < 0) || (fxpos > (WIDTH-20)))
	fstatus = 0;
      } // End update Fabbri
    /**********************************************************************/
    /*                                                                    */
    /* END FABBRI                                                         */
    /*                                                                    */
    /**********************************************************************/

    /**********************************************************************/
    /*                                                                    */
    /* START STUDENT                                                      */
    /*                                                                    */
    /**********************************************************************/
    if (RAND % 100 < 2)
      {
      for (loop = 0; loop < 20; loop++)
	{
	if (students[loop].active == 0)
	  { 
	  // Add a new student!

	  students[loop].active = 1;
	  students[loop].x = 10 + (RAND % (WIDTH - 30));
	  students[loop].y = 0;
	  students[loop].face = RAND % 2;
	  loop = 20;
	  }
	}
      }
    // Need to move each of them
    for (loop = 0; loop < 20; loop++)
      {
      if (students[loop].active == 1)
	{
	if (map[(students[loop].y + 20)/30][(students[loop].x+10)/30] == 1)
	  {
	  if (RAND % 100 < 1)
	    {
	    students[loop].face = abs (students[loop].face - 1);
	    }
	  if (students[loop].face == 0)
	    {
	    students[loop].x += 2;
	    }
	  else
	    {
	    students[loop].x -= 2;
	    }
	  }
	else
	  {
	  students[loop].y += 5;
	  }
	// If the student gets to the edge of the screen
	if ((students[loop].x < 0) || (students[loop].x > WIDTH -15))
	  {
	  students[loop].active = 0;
	  }
	
	// Students drop their books
	if (((RAND % 500) < 1)&&
	    (map[(students[loop].y)/30][(students[loop].x+10)/30] == 0))
	  {
	  map[(students[loop].y)/30][(students[loop].x+10)/30] = 2;
	  do_sounds(sounds[bookdrop]);
	  }
	
	// Students steal the cable tester
	if (map[(students[loop].y)/30][(students[loop].x+10)/30] == 4)
	  {
	  do_sounds(sounds[evil_laugh]);
	  map[(students[loop].y)/30][(students[loop].x+10)/30] = 0;
	  }
	// If we land on a student
	if ((yvel > 0) && ((xpos+25 > students[loop].x) && 
			   (xpos+ 5 < students[loop].x + 10) &&
			   (ypos+30 > students[loop].y - 4) &&
			   (ypos+30 < students[loop].y + 5)))
	  {
	  students[loop].active = 0;
	  if (real == 1)
	    p1score += 50;
	  do_sounds(sounds[squish]);
	  }

	}
      } // For each student
    /**********************************************************************/
    /*                                                                    */
    /* END STUDENT                                                        */
    /*                                                                    */
    /**********************************************************************/

    /**********************************************************************/
    /*                                                                    */
    /* START BOGER                                                        */
    /*                                                                    */
    /**********************************************************************/
    // Does Boger appear?
    if ((bogerstat == 0) && ((RAND % 100) < 2))
      {
      bogerstat = 1;
      bogerx = 10 + (RAND % (WIDTH - 30));
      bogerhouse = bogerx - 37;
      bogery = -28;
      }
    // Boger is going up
    if (bogerstat == 2)
      {
      bogery -= 2;
      if (bogery == -30) bogerstat = 0;
      }
    // Boger sees us
    if (((bogerstat == 2)||(bogerstat == 1))
	&& ((bogerx > xpos-50 ) && (bogerx+ 30 < xpos+75)))
      {
      bogery += 2;
      bogerstat = 3;
      do_sounds(sounds[falling]);
      }
    // Boger is going down
    if (bogerstat == 1)
      {
      bogery += 2;
      if (bogery == 0) 	bogerstat = 2;
      }

    // Boger is falling
    if (bogerstat == 3)
      {
      if ((twigpower == 0)&&
	  (testcollide(xpos+15, ypos+15, 14, bogerx+12, bogery+15, 12)))
	{
	do_death();
	lives--;
	if (lives == 0) 
	  {
	  if (do_end_of_game() == 0)	  return;
	  }
	Mix_PlayMusic(musics[bg], -1);
	// Reset level
	restart_level();
	} // End dead
      // Update position
      bogery += 8;
      if (bogery > HEIGHT + OFFSET)
	bogerstat = 0;
      if (bogerx < xpos) bogerx += 3;
      if (bogerx > xpos) bogerx -= 3;
      }// End Boger
    /**********************************************************************/
    /*                                                                    */
    /* END BOGER                                                          */
    /*                                                                    */
    /**********************************************************************/


    if (p1score > 1000000)  
	{
	p1score -= 1000000; // Roll score!
	newlife = 1; // Roll the newlife counter too
	}

    // If you got a new life
    if (p1score > 15000*newlife)
      {
      lives++;
      if (lives > 15) lives = 15; // Max of 15 extra lives for savefile
      newlife++;
      do_sounds(sounds[life]);
      }
    // New hiscore
    if (p1score > hiscore) hiscore = p1score;

    draw_screen(real);
    
    // TwigPower decays over time.  Need more diet cola
    twigpower-=2;
    if (twigpower < 0) twigpower = 0;

    /**********************************************************************/
    /*                                                                    */
    /* START KEYSTATE                                                     */
    /*                                                                    */
    /**********************************************************************/
    SDL_PumpEvents();
    keystate = SDL_GetKeyState(NULL);

    // Since left and right could be held down, we use getkeystate for them.
    if (((keystate[keys[LEFTKEY]])&&(real == 1)) ||
	((real == 0) && (demo_play[teaser_tick] & 1)))
      {
      xpos -= 5;
      if (xpos < 0) xpos = 0;  // Not too far past edge of screen
      xface = 0;
      }

    if (((keystate[keys[RIGHTKEY]])&&(real == 1)) ||
	((real == 0) && (demo_play[teaser_tick] & 2)))	
      {
      xpos += 5;
      if (xpos >(WIDTH-30)) xpos= WIDTH-30; // Not too far past edge of screen
      xface = 1;
      }

    if ((real == 0) && (demo_play[teaser_tick] & 4))
      {
      do_sounds(sounds[jumpping]);
      jump = 1;
      yvel = JUMP;	    
      }


    // These keys only trip once per press.  Can't be held down!
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[QUITKEY])
	    exit(0);
	  if (((event.key.keysym.sym == keys[JUMPKEY])&&
	       ((jump==0)||(twigpower>0)))
	      && (real == 1))
	    {
	    do_sounds(sounds[jumpping]);
	    jump = 1;
	    yvel = JUMP;	    
	    }
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    if (real == 0) return;
	    }
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }

	  if (event.key.keysym.sym == keys[PAUSEKEY])
	    {
	    pause_mode();
	    }
	}
      } // End Event poll

    SDL_Delay(50);  // 20 Frames per second


    tick = (tick+1)%2520; // Animation clock
    teaser_tick++;

    if ((real == 0)&&(teaser_tick == 360)) return;
    } // End while
 
  } // End play_game

/*****************************************************************************
 * 
 * Insert_coin teaser loop 
 *
 *****************************************************************************/
void insert_coin(int type)
  {
  int loop;
  SDL_Event event;
  SDL_Rect rect;
  SDL_Rect pos;

  for (loop = 0; loop < 100; loop++)
    {
    if (type == 1)  // Only show this for the second Insert Coin!
      {
      rect.x = 60; rect.y =45; rect.w = 30; rect.h = 380;
      SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format,0,0,0));

      pos.x = 60;
      rect.y = 0; rect.w = 30; rect.h = 30; 
      
      rect.x = 0; pos.y = 45;
      SDL_BlitSurface(book, &rect, screen, &pos);
      
      rect.x =(tick % 6)* 30; pos.y = 85; 
      SDL_BlitSurface(cola, &rect, screen, &pos);
      
      rect.x =(tick/10)%6 * 30; pos.y = 125;
      SDL_BlitSurface(cable, &rect, screen, &pos);
      
      rect.x = 20 * (tick % 8); rect.w = 20;rect.h = 20;pos.x = 65;pos.y = 165;
      SDL_BlitSurface(student, &rect, screen, &pos);
      
      rect.x = 30 * ((tick/23)%2);rect.w =30;rect.h= 30;pos.x =60; pos.y = 315;
      SDL_BlitSurface(player, &rect, screen, &pos);
      
      rect.x = 30 * ((tick/29) % 2); pos.y = 355;
      SDL_BlitSurface(fabbri, &rect, screen, &pos);
      
      rect.x = 0; rect.w = 25; pos.x = 62; pos.y = 395;
      SDL_BlitSurface(boger, &rect, screen, &pos);
      }


    // Start by blanking the screen
    rect.x = 140; rect.y = 210; rect.w = 305; rect.h = 50;
    SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format,0,0,0));
    big_xprint("Insert Coin", 140, 200);
    SDL_UpdateRect(screen,0,0,0,0);
    
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    return;
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }

    SDL_Delay(50);
    tick = (tick+1)%2520;
   
    }

  }
  

  
/****************************************************************************
 *
 * point_screen - Show what the game objects are and what they are for!
 * 
 ****************************************************************************/
void point_screen()
  {
  int loop;
  SDL_Event event;
  SDL_Color c;

  SDL_Rect rect;
  SDL_Rect pos;
  char str[100];
  
 
  c.r = 255; c.g = 255; c.b = 255;
  for (loop = 0; loop < 100; loop++)
    {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0));
    FixedPrint(screen, 140, 50, " 10     CSCI 221 Textbook      Dropped by the students", &c);
    FixedPrint(screen, 140, 90, "100     Diet Cola              Gives TwigMan Super-Mega-Bonus-TwigPower", &c);
    FixedPrint(screen, 140,130, "300     Cable Tester           Collect enough for each level", &c);
    FixedPrint(screen, 140,170, " 50     Student                They steal the Cable Testers", &c);
    FixedPrint(screen, 140,320, "        TwigMan                Hero of CSCI 221 Lab", &c);
    FixedPrint(screen, 140,360, "        Dr. Fabbri             Steals books he didn't write", &c);
    FixedPrint(screen, 140,400, "        Professor Boger        Looking for a soft place to land", &c);

    pos.x = 60;
    rect.y = 0; rect.w = 30; rect.h = 30; 

    rect.x = 0; pos.y = 45;
    SDL_BlitSurface(book, &rect, screen, &pos);

    rect.x =(tick % 6)* 30; pos.y = 85; 
    SDL_BlitSurface(cola, &rect, screen, &pos);
    
    rect.x =(tick/10)%6 * 30; pos.y = 125;
    SDL_BlitSurface(cable, &rect, screen, &pos);

    rect.x = 20 * (tick % 8); rect.w = 20; rect.h = 20;pos.x = 65; pos.y = 165;
    SDL_BlitSurface(student, &rect, screen, &pos);

    rect.x = 30 * ((tick/23)%2);rect.w =30;rect.h = 30;pos.x = 60; pos.y = 315;
    SDL_BlitSurface(player, &rect, screen, &pos);

    rect.x = 30 * ((tick/29) % 2); pos.y = 355;
    SDL_BlitSurface(fabbri, &rect, screen, &pos);
    
    rect.x = 0; rect.w = 25; pos.x = 62; pos.y = 395;
    SDL_BlitSurface(boger, &rect, screen, &pos);
    

    sprintf(str, "P1: %06d", p1score);
    little_xprint(str, 1, 0);
    
    sprintf(str, "High Score: %06d", hiscore);
    little_xprint(str, WIDTH/3, 0);

    SDL_UpdateRect(screen,0,0,0,0);





    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    return;
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    tick = (tick+1)%2520; 
    }

  }


/****************************************************************************
 *
 * p1_start - Once they have entered a coin, but not pressed p1 start.  This 
 * holds them until they do.
 * 
 ****************************************************************************/
void p1_start()
  {
  int loop;
  SDL_Event event;
  SDL_Rect rect;
  SDL_Rect pos;

  while(1)
    {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0));
    big_xprint("Press", 240, 170);
    big_xprint("Player 1", 195, 250);
    big_xprint("Start", 240, 330);

    little_xprint("New TwigMan each 15000 points", 165, 430);

    for (loop = 0; loop < 16; loop++)
      {
      rect.x = 20*(tick % 8); rect.w = 20; rect.h = 20; rect.y = 0;
      pos.x = -20+(loop * 40) + (tick%40); pos.y = 160;
      SDL_BlitSurface(student, &rect, screen, &pos);
      pos.x = -20+(loop * 40) + (40-(tick%40));rect.y = 20; pos.y = 240;
      SDL_BlitSurface(student, &rect, screen, &pos);
      pos.x = -20+(loop * 40) + (tick%40);rect.y = 0; pos.y = 320;
      SDL_BlitSurface(student, &rect, screen, &pos);
      pos.x = -20+(loop * 40) + (40-(tick%40));rect.y = 20; pos.y = 400;
      SDL_BlitSurface(student, &rect, screen, &pos);
      
      }

    rect.x = 0; rect.y = 0; rect.w = 300; rect.h = 130;
    pos.x = 150;pos.y = 10;
    SDL_BlitSurface(logo, &rect, screen, &pos);
    SDL_UpdateRect(screen,0,0,0,0);
    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[STARTKEY])
	    return;
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    tick = (tick+1)%2520; 
    }

  }

/****************************************************************************
 * 
 * teaser - Main logo screen!
 * 
 ****************************************************************************/
void teaser()
  {
  int loop;
  SDL_Event event;
  SDL_Rect rect;
  SDL_Rect pos;
  char str[100];

  for (loop = 0; loop < 100; loop++)
    {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0));
    rect.x = 0; rect.y = 0; rect.w = 300; rect.h = 130;
    pos.x = 150;pos.y = 50;
    SDL_BlitSurface(logo, &rect, screen, &pos);
    little_xprint("Created by Dr. Evan L. Schemm", 155, 300);
    little_xprint("Copyright 2007", 228, 330);
    little_xprint("Department of Mathematics and Computer Science", 70, 360);
    little_xprint("Lake Superior State University", 150, 390);

    sprintf(str, "P1: %06d", p1score);
    little_xprint(str, 1, 0);
    
    sprintf(str, "High Score: %06d", hiscore);
    little_xprint(str, WIDTH/3, 0);
    
    SDL_UpdateRect(screen,0,0,0,0);


    while (SDL_PollEvent(&event) > 0)
      {
      switch(event.type)
	{
	case SDL_QUIT:
	  exit(0);
	case SDL_KEYDOWN:
	  if (event.key.keysym.sym == keys[COINKEY])
	    {
	    coins++;
            if (coins > 127) coins = 127;
	    do_sounds(sounds[money]);
	    return;
	    }
	  if (event.key.keysym.sym == keys[QUITKEY]) exit(0);
	  if (event.key.keysym.sym == keys[FSKEY])
	    {
	    screen_flags = screen_flags ^ SDL_FULLSCREEN;
	    change_screen(WIDTH, HEIGHT+OFFSET);
	    }
	}
      }
    SDL_Delay(50);
    }

  }



// Main!  Left to the student to figure out!
int main(int argc, char *argv[])
  {
  SDL_Surface* temp;
  FILE* test;

  init_SDL_stuff(); // Do the SDL stuff

// Need to set the location of the savefile!
#ifdef WIN32
  sprintf(SAVE_FILE,"%s%s/twigman.save",getenv("HOMEDRIVE"),getenv("HOMEPATH"));
#endif
#ifndef WIN32
  sprintf(SAVE_FILE, "%s/.twigman.save", getenv("HOME"));
#endif


  // Still need to load the surface, since it's not from a file, XPM
  // uses a slightly different command!
  temp  = IMG_ReadXPMFromArray(NewerTwig3_xpm);
  player = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(player,SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(book_xpm);
  book = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(book,SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(fabbri_xpm);
  fabbri = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(fabbri, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(cola_xpm);
  cola = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(cola, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(cable_xpm);
  cable = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(cable, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(man_xpm);
  student = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(student, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));
  
  temp = IMG_ReadXPMFromArray(boger30_xpm);
  boger = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(boger, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(roof_xpm);
  roof = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(roof, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_Load(LOGO_FILE);
  logo = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(logo, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format, 255,0,255));

  temp = IMG_ReadXPMFromArray(level_xpm);
  levelglyph = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(levelglyph, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(grider_xpm);
  brick = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(brick, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(BigBoger2_xpm);
  BigBoger = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(BigBoger, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_Load(IBM_FILE);
  ibm_360 = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(ibm_360, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(rock_xpm);
  rock = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(rock, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(crater_xpm);
  crater = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(crater, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_Load(ROAD_FILE);
  road = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(road, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format, 255,0,255));

  temp = IMG_ReadXPMFromArray(signs_xpm);
  signs = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(signs, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(house_xpm);
  house = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(house, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(trucks_xpm);
  trucks = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(trucks, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(misc_xpm);
  misc = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(misc, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(BigTwig2_xpm);
  BigTwig = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(BigTwig, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(BigSchemm2_xpm);
  BigSchemm = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(BigSchemm, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(mover_xpm);
  movers = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(movers, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));
  
  temp = IMG_ReadXPMFromArray(cats_xpm);
  cats = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(cats, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));
  
  temp = IMG_ReadXPMFromArray(BigFabbri2_xpm);
  BigFabbri = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(BigFabbri, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_Load(CASET_FILE);
  caset = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(caset, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format, 255,0,255));

  temp = IMG_Load(ZOMBIE_FILE);
  zombie = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(zombie, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(dean_xpm);
  dean = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(dean, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(game_items_xpm);
  pacitems = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(pacitems, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));




  // Dinosaur clipart in the public domain as certified by fundraw.com
  // Can be found at www.fundraw.com/clipart/categories/00000139
  // Modified by Dr. Evan L. Schemm for use in this program
  temp = IMG_Load(DINO_FILE);
  dino1 = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(dino1, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(dino2_xpm);
  dino2 = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(dino2, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_ReadXPMFromArray(dino3_xpm);
  dino3 = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(dino3, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));

  temp = IMG_Load(LITTLEFONT_FILE);
  littlefont = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(littlefont, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format, 255,0,255));

  temp = IMG_Load(BIGFONT_FILE);
  bigfont = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);
  SDL_SetColorKey(bigfont, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,255,0,255));



  // Downloaded from flashkit.com

  sounds[endgame]=Mix_LoadWAV(ENDGAME_SOUND);//Made by: Ilhan Bayrak
  sounds[evil_laugh]=Mix_LoadWAV(EVIL_LAUGH_SOUND);//Made by: Wiretrip
  sounds[money] = Mix_LoadWAV(MONEY_SOUND);// Made by: Colin Dodd
  sounds[death] = Mix_LoadWAV(DEATH_SOUND); // Made by: Doron Add

  sounds[can_open] = Mix_LoadWAV(CAN_OPEN_SOUND);// Made by SodaBush

  sounds[falling] = Mix_LoadWAV(FALLING_SOUND);// Made by Deflux Design
  sounds[steal] = Mix_LoadWAV(STEAL_SOUND); // Made by Amit Patel
  sounds[intermission] = Mix_LoadWAV(INTERMISSION_SOUND); // Made by A Algien
  sounds[crash] = Mix_LoadWAV(CRASH_SOUND); // Made by Adrian Gallant
  sounds[thud] = Mix_LoadWAV(THUD_SOUND); // Made by Splinteh
  sounds[meow] = Mix_LoadWAV(MEOW_SOUND); // Made by Starmanltd
  sounds[purr] = Mix_LoadWAV(PURR_SOUND); // Made by Richard Harding
  sounds[gun] = Mix_LoadWAV(GUN_SOUND); // Made by Maboroshi Tsukuru
  sounds[life] = Mix_LoadWAV(LIFE_SOUND); // Made by Crow xxx
  // The following sounds made by:  Sith Master
  sounds[appear]=Mix_LoadWAV(APPEAR_SOUND);
  sounds[get_tester] = Mix_LoadWAV(GET_TESTER_SOUND);
  sounds[get_book] = Mix_LoadWAV(GET_BOOK_SOUND);

 
  // Downloaded from www.grsites.com/sounds/cartoon008.shtml (cartoon148.wav)
  sounds[flower] = Mix_LoadWAV(FLOWER_SOUND);
  // Downloaded from www.grsites.com/sounds/cartoon009.shtml (cartoon172.wav)
  sounds[squish] = Mix_LoadWAV(SQUISH_SOUND);
  // Downloaded from www.grsites.com/sounds/horror001.html (horror004.wav)
  sounds[roar] = Mix_LoadWAV(ROAR_SOUND);
  // Downloaded from simplythebest.net/sounds/WAV/cartoon_wavs.html
  sounds[jumpping] = Mix_LoadWAV(JUMPING_SOUND);
  // Downloaded from www.acoustica.com/mp3-audio-mixer
  sounds[drive] = Mix_LoadWAV(DRIVE_SOUND);
  // Downloaded from freesound.iua.upf.edu
  sounds[bookdrop] = Mix_LoadWAV(BOOKDROP_SOUND);


  // Downloaded from flashkit.com  Made by: Nemo
  musics[bg] = Mix_LoadMUS(BG_MUSIC);
  musics[inter1] = Mix_LoadMUS(INTER1_MUSIC);

  tick = 0;  // Start animation clock
  coins = 0; // No coins at power on


  // if savefile exists, then run from there
  if ((test = fopen(SAVE_FILE, "r")) != NULL)
    {
    fclose(test);
    play_game(2);
    }
  

  // Main game loop
  while(1)
    {
    // Teaser screen
    if (coins == 0)      teaser();
      
    // Insert coin
    if (coins == 0)      insert_coin(0);

    // Scoring screen
    if (coins == 0)      point_screen();

    // Insert coin
    if (coins == 0)      insert_coin(1);
    
    // Play game demo
    if (coins == 0)      play_game(0);

    // If there is a coin
    while (coins > 0)
      {
      // Wait for player 1 start
      p1_start();
      // Play
      coins--;
      play_game(1);
      
      }
    } // End main game while
  } // End main

